{*******************************************************}
{                                                       }
{         CWWinProcs  component                         }
{  Copyright (c) 1997, 1998 Cordaware                   }
{             FREEWARE                                  }
{                                                       }
{                                                       }
{*******************************************************}


unit CWWinProcs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

{$R *.res}


  TCWMsgEvent = procedure(Sender: TControl; var Message: TMessage) of object;
  TCWWinEvent = procedure(Sender: TControl; var Message: TMessage;
                                            var Action: Boolean) of object;


{TWinProcSection}

 TCWWinProcs = class;

 TWinProcSection = class(TCollectionItem)
  private
    FControl: TControl;
    FWindowProc: TWndMethod;
   procedure SetControl(Value: TControl);
  public
   constructor Create(Collection: TCollection);override;
   destructor Destroy; override;
   property Control: TControl read FControl write SetControl;
   procedure WndProc(var Message: TMessage);
  end;

{TWinProcSections}

  TWinProcSections = class(TCollection)
  private
    FCWWinProcs: TCWWinProcs;
    procedure SetItem(Index: Integer; Value: TWinProcSection);
    function GetItem(Index: Integer): TWinProcSection;
  public
    function Add(Control: TControl): TWinProcSection;
    function IndexofControl(Value: TObject): Integer;
    function Controls(Index: Integer): TControl;
    property Items[Index: Integer]: TWinProcSection read GetItem write SetItem; default;
  end;


  TCWWinProcs = class(TComponent)
  private
    { Private-Deklarationen }
   FBeforeMsg: TCWMsgEvent;
   FAfterMsg: TCWMsgEvent;
   FOnAdd: TCWWinEvent;
   FOnDelete: TCWWinEvent;
   FWinProcSections: TWinProcSections;
   FNoUseControl: TStrings;
   procedure SetNoUseControl(Value: TStrings);
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   function UseControl(AControl: TControl): Boolean;
   procedure Init(AControl: TControl);
  published
    { Published-Deklarationen }
   property OnBeforeMsg: TCWMsgEvent read FBeforeMsg write FBeforeMsg;
   property OnAfterMsg: TCWMsgEvent read  FAfterMsg write  FAfterMsg;
   property OnAdd: TCWWinEvent read FOnAdd write FOnAdd;
   property OnDelete: TCWWinEvent read FOnDelete write FOnDelete;
   property NoUseControl: TStrings read FNoUseControl write SetNoUseControl;
    { for example TForm }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Cordaware', [TCWWinProcs]);
end;

{WinProcSection}

constructor TWinProcSection.Create(Collection: TCollection);
begin
inherited Create(Collection);
FControl := Nil;
end;

destructor TWinProcSection.Destroy;
begin
Control := Nil;
inherited Destroy;
end;

procedure TWinProcSection.SetControl(Value: TControl);
begin
if Value <> FControl then
   begin
    if FControl <> Nil then
     FControl.WindowProc := FWindowProc;
    FControl := Value;
    if Value <> Nil then
     begin
      FWindowProc := Value.WindowProc;
      Value.WindowProc := WndProc;
     end;
   end;
end;

procedure TWinProcSection.WndProc(var Message: TMessage);
var CWWinProcs: TCWWinProcs;
    Index: Integer;
    Action: Boolean;
begin
      if Assigned(Collection) and
         Assigned(TWinProcSections(Collection).FCWWinProcs) then
       CWWinProcs := TWinProcSections(Collection).FCWWinProcs else
       CWWinProcs := Nil;


      if (Message.Msg = CM_CONTROLCHANGE) and (Message.LParam = 1) and
         Assigned(TControl(Message.WParam)) then
       begin
         Action := true;
         if Assigned(CWWinProcs) and Assigned(CWWinProcs.FOnAdd) then
             CWWinProcs.FOnAdd(TControl(Message.WParam), Message, Action);
         if Action then
         TWinProcSections(Collection).Add(TControl(Message.WParam));
       end;

      if Assigned(CWWinProcs) and Assigned(CWWinProcs.FBeforeMsg) then
                  CWWinProcs.FBeforeMsg(Control, Message);


      FWindowProc(Message); //Message

      if Assigned(CWWinProcs) and Assigned(CWWinProcs.FAfterMsg) then
                  CWWinProcs.FAfterMsg(Control, Message);

      if (Message.Msg = CM_CONTROLCHANGE) and (Message.LParam = 0) and
         Assigned(TControl(Message.WParam)) then
       begin
         Index := TWinProcSections(Collection).IndexofControl(TControl(Message.WParam));
         if Index <> - 1 then
          begin
           Action := true;
           if Assigned(CWWinProcs) and Assigned(CWWinProcs.FOnDelete) then
              CWWinProcs.FOnDelete(TControl(Message.WParam), Message, Action);
           if Action then
           TWinProcSections(Collection).Items[Index].free;
          end;
       end;
end;

{WinProcSections}

function TWinProcSections.Add(Control: TControl): TWinProcSection;
var I: Integer;
begin

  Result := Nil; 
  if not(FCWWinProcs.UseControl(Control))  then
   exit;

  if IndexofControl(Control) = - 1 then
   begin
    Result := TWinProcSection(inherited Add);
    Result.Control := Control;
   end else begin Result := Nil;exit;end;

   for I := 0 to Control.ComponentCount - 1 do
     if (Control.Components[I] is TControl) then
        Add(TControl(Control.Components[I]));

   if Control is TWinControl then
    for I := 0 to TWinControl(Control).ControlCount - 1 do
        Add(TWinControl(Control).Controls[I]);

end;

function TWinProcSections.GetItem(Index: Integer): TWinProcSection;
begin
  Result := TWinProcSection(inherited GetItem(Index));
end;

procedure TWinProcSections.SetItem(Index: Integer; Value: TWinProcSection);
begin
  inherited SetItem(Index, Value);
end;

function TWinProcSections.IndexofControl(Value: TObject): Integer;
var I: Integer;
begin
   Result := -1;
   for I := 0 to count - 1 do
    if TObject(Items[I].FControl) = Value then
     begin
      Result := I;
      break;
     end;
end;

function TWinProcSections.Controls(Index: Integer): TControl;
begin
      Result := Items[Index].Control
end;


{ TCWWinProcs }

constructor TCWWinProcs.Create(AOwner: TComponent);
begin
      inherited;
      FNoUseControl := TStringList.create;
      FBeforeMsg := Nil;
      FAfterMsg := Nil;
      FOnAdd := Nil;
      FOnDelete := Nil;
      FWinProcSections := TWinProcSections.create(TWinProcSection);
      FWinProcSections.FCWWinProcs := Self;
      FNoUseControl.Add('TStretchHandles');
      if AOwner is TControl then
      FWinprocSections.Add(TControl(AOwner));
end;

destructor TCWWinProcs.Destroy;
begin
      FBeforeMsg := Nil;
      FAfterMsg := Nil;
      FOnAdd := Nil;
      FOnDelete := Nil;
      FWinProcSections.free;
      FNoUseControl.free;
      inherited;
end;

function TCWWinProcs.UseControl(AControl: TControl): Boolean;
var AName,AClassName: String;
begin
  Result := false;

  if AControl = nil then exit;

  try
   AName := AControl.Name;
  except
  end;

  try
   AClassName := AControl.ClassName;
  except
  end;

  Result := (FNoUseControl.Indexof(AName) = -1) and
            (FNoUseControl.Indexof(AClassName) = -1);

end;

procedure TCWWinProcs.SetNoUseControl(Value: TStrings);
begin
  FNoUseControl.Assign(Value);
end;

procedure TCWWinProcs.Init(AControl: TControl);
begin
      FWinProcSections.clear;
      if Assigned(AControl) then
      FWinProcSections.Add(AControl);
end;

end.
