unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls,Handles,DB, DBTables,
  UQuerys, UZoom, UCWReport, DBCtrls;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileCloseItem: TMenuItem;
    Window1: TMenuItem;
    Help1: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    WindowCascadeItem: TMenuItem;
    WindowTileItem: TMenuItem;
    WindowArrangeItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDialog: TOpenDialog;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    WindowMinimizeItem: TMenuItem;
    SpeedPanel: TPanel;
    OpenBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    PrintBtn: TSpeedButton;
    PrinttoBtn: TSpeedButton;
    BandBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    StatusBar: TStatusBar;
    SaveDialog: TSaveDialog;
    ColorDialog: TColorDialog;
    PrintItem: TMenuItem;
    N2: TMenuItem;
    PrinttoItem: TMenuItem;
    PrintDialog1: TPrintDialog;
    Insert1: TMenuItem;
    BandItem: TMenuItem;
    FieldItem: TMenuItem;
    FieldBtn: TSpeedButton;
    ArrowBtn: TSpeedButton;
    View1: TMenuItem;
    NextReportPageItem: TMenuItem;
    PriviousReportPageItem: TMenuItem;
    N3: TMenuItem;
    ZoomItem: TMenuItem;
    Datasets: TMenuItem;
    TextItem: TMenuItem;
    TextBtn: TSpeedButton;
    FontDialog: TFontDialog;
    RefreshBtn: TSpeedButton;
    DSBtn: TSpeedButton;
    N5: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FileNewItemClick(Sender: TObject);
    procedure WindowCascadeItemClick(Sender: TObject);
    procedure UpdateMenuItems(Sender: TObject);
    procedure WindowTileItemClick(Sender: TObject);
    procedure WindowArrangeItemClick(Sender: TObject);
    procedure FileCloseItemClick(Sender: TObject);
    procedure FileOpenItemClick(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
    procedure FileSaveItemClick(Sender: TObject);
    procedure FileSaveAsItemClick(Sender: TObject);
    procedure CutItemClick(Sender: TObject);
    procedure CopyItemClick(Sender: TObject);
    procedure PasteItemClick(Sender: TObject);
    procedure WindowMinimizeItemClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure HelpAboutItemClick(Sender: TObject);
    procedure PrinttoItemClick(Sender: TObject);
    procedure PrintItemClick(Sender: TObject);
    procedure BandBtnClick(Sender: TObject);
    procedure ArrowBtnClick(Sender: TObject);
    procedure FieldBtnClick(Sender: TObject);
    procedure NextReportPageClick(Sender: TObject);
    procedure PriviousReportPageClick(Sender: TObject);
    procedure ZoomClick(Sender: TObject);
    procedure DatasetsClick(Sender: TObject);
    procedure TextBtnClick(Sender: TObject);
    procedure TextItemClick(Sender: TObject);
    procedure BandItemClick(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure DSBtnClick(Sender: TObject);
  private
    { Private declarations }
    procedure CreateMDIChild(const Name: string);
    procedure ShowHint(Sender: TObject);
  public
    { Public declarations }
   ZoomForm: TFMZoom;
   QueryForm: TFMQuerys;
   InsertComp: TComponent;
   procedure Save(AFileName: String);
   procedure AppException(Sender: TObject; E: Exception);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses ChildWin, About;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
  Application.OnException := AppException;
  Screen.OnActiveFormChange := UpdateMenuItems;
  QueryForm := TFMQuerys.Create(Self);
  ZoomForm := TFMZoom.create(self);


  RegisterClasses([TMdiChild, TStretchHandles, TDatasource, TQuery]);
  RegisterClasses([TCWBand, TStaticText, TDBEdit, TDBImage, TDBMemo]);
end;

procedure TMainForm.AppException(Sender: TObject; E: Exception);
begin
  if CompareText(E.Classname,'EAccessViolation') <> 0 then  // no show AccessViolation
  Application.ShowException(E);
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := Application.Hint;
end;

procedure TMainForm.CreateMDIChild(const Name: string);
var
  Child: TMDIChild;
begin
  { create a new MDI child window }
  Child := TMDIChild.Create(Application);
  Child.Caption := Name;
end;

procedure TMainForm.FileNewItemClick(Sender: TObject);
begin
  CreateMDIChild('REPORT' + IntToStr(MDIChildCount + 1));
end;

procedure TMainForm.FileOpenItemClick(Sender: TObject);
var InStream: TFileStream;
    OutStream: TMemoryStream;
begin
  if OpenDialog.Execute then
   begin
    InStream := TFileStream.create(OpenDialog.FileName, fmOpenRead);
    OutStream := TMemoryStream.create;
   try
    CreateMDIChild(OpenDialog.FileName);
    
    while ActiveMDIChild.ComponentCount > 0 do
     ActiveMDIChild.Components[0].free;

    ObjectTextToResource(InStream, OutStream);
    OutStream.Position := 0;
    OutStream.ReadComponentRes(ActiveMDIChild);
    ActiveMDIChild.Caption := OpenDialog.FileName;
    
   finally
    InStream.Free;
    OutStream.Free;
   end;
   end;
end;

procedure TMainForm.FileCloseItemClick(Sender: TObject);
begin
  if ActiveMDIChild <> nil then
    ActiveMDIChild.Close;
end;

procedure TMainForm.FileSaveItemClick(Sender: TObject);
begin
  if not FileExists(ActiveMDIChild.Caption) then
   FileSaveAsItemClick(Sender) else
   Save(ActiveMDIChild.Caption);
end;

procedure TMainForm.FileSaveAsItemClick(Sender: TObject);
var Stream: TFileStream;
    AName: String;
begin
  if SaveDialog.Execute then
   Save(SaveDialog.FileName);
end;

procedure TMainForm.Save(AFileName: String);
var OutStream: TFileStream;
    InStream: TMemoryStream;
begin
    InStream := TMemoryStream.create;
    OutStream := TFileStream.create(AFileName, fmCreate);
   try
    ActiveMDIChild.ActiveControl := Nil; //
    TMDIChild(ActiveMDIChild).Handles.Detach;
    InStream.WriteComponentRes(ActiveMDIChild.classname,ActiveMDIChild);
    InStream.Position := 0;
    ObjectResourceToText(InStream, OutStream);
    ActiveMDIChild.Caption := AFileName;
   finally
    InStream.free;
    OutStream.Free;
   end;
end;

procedure TMainForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.CutItemClick(Sender: TObject);
begin
  {cut selection to clipboard}
end;

procedure TMainForm.CopyItemClick(Sender: TObject);
begin
  {copy selection to clipboard}
end;

procedure TMainForm.PasteItemClick(Sender: TObject);
begin
  {paste from clipboard}
end;

procedure TMainForm.WindowCascadeItemClick(Sender: TObject);
begin
  Cascade;
end;

procedure TMainForm.WindowTileItemClick(Sender: TObject);
begin
  Tile;
end;

procedure TMainForm.WindowArrangeItemClick(Sender: TObject);
begin
  ArrangeIcons;
end;

procedure TMainForm.WindowMinimizeItemClick(Sender: TObject);
var
  I: Integer;
begin
  { Must be done backwards through the MDIChildren array }
  for I := MDIChildCount - 1 downto 0 do
    MDIChildren[I].WindowState := wsMinimized;
end;

procedure TMainForm.UpdateMenuItems(Sender: TObject);
begin
  FileCloseItem.Enabled := MDIChildCount > 0;
  FileSaveItem.Enabled := MDIChildCount > 0;
  FileSaveAsItem.Enabled := MDIChildCount > 0;
  TextItem.Enabled := MDIChildCount > 0;
  PriviousReportPageItem.Enabled := MDIChildCount > 0;
  NextReportPageItem.Enabled := MDIChildCount > 0;
  PrintItem.Enabled := MDIChildCount > 0;
  PrinttoItem.Enabled := MDIChildCount > 0;
  ZoomItem.Enabled := MDIChildCount > 0;
  SaveBtn.Enabled := MDIChildCount > 0;
  RefreshBtn.Enabled := MDIChildCount > 0;
  DSBtn.Enabled := MDIChildCount > 0;
  BandItem.Enabled := MDIChildCount > 0;
  FieldItem.Enabled := MDIChildCount > 0;
  BandBtn.Enabled := MDIChildCount > 0;
  FieldBtn.Enabled := MDIChildCount > 0;
  TextBtn.Enabled := MDIChildCount > 0;
  ArrowBtn.Enabled := MDIChildCount > 0;
  PrintBtn.Enabled := MDIChildCount > 0;
  PrinttoBtn.Enabled := MDIChildCount > 0;
  Datasets.Enabled := MDIChildCount > 0;
  WindowCascadeItem.Enabled := MDIChildCount > 0;
  WindowTileItem.Enabled := MDIChildCount > 0;
  WindowArrangeItem.Enabled := MDIChildCount > 0;
  WindowMinimizeItem.Enabled := MDIChildCount > 0;
  QueryForm.UpdateQuery(ActiveMDIChild);
  ZoomForm.UpdateZoom;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Screen.OnActiveFormChange := nil;
  Application.OnHint := nil;
  Application.OnException := nil;
end;

procedure TMainForm.HelpAboutItemClick(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TMainForm.PrinttoItemClick(Sender: TObject);
var
  I, Start, Stop: Integer;
begin

PrintDialog1.Options := [poPageNums, poSelection];
PrintDialog1.FromPage := 1;
PrintDialog1.MinPage := 1;
PrintDialog1.ToPage := TMdiChild(ActiveMDIChild).CWReport.PageCount;
PrintDialog1.MaxPage := TMdiChild(ActiveMDIChild).CWReport.PageCount;

if PrintDialog1.Execute then
  begin
  with PrintDialog1, TMdiChild(ActiveMDIChild) do
  begin
    if PrintRange = prAllPages then
      begin

      Start := MinPage;
      Stop := MaxPage;
      end
    else if PrintRange = prSelection then
      begin
      Start := CWReport.PageIndex;
      Stop := Start;
      end
    else  { PrintRange = prPageNums }
      begin
      Start := FromPage ;
      Stop := ToPage;
      end;
  end;
  TMdiChild(ActiveMDIChild).CWReport.Print(Start,Stop);
  end;
end;


procedure TMainForm.PrintItemClick(Sender: TObject);
begin
 with TMdiChild(ActiveMDIChild) do
  CWReport.Print(1,CWReport.PageCount);
end;

procedure TMainForm.BandBtnClick(Sender: TObject);
begin
If TSpeedButton(Sender).Down then
   Screen.Cursor := crDrag else
   Screen.Cursor := crDefault;
end;

procedure TMainForm.ArrowBtnClick(Sender: TObject);
begin
Screen.Cursor := crDefault;
end;

procedure TMainForm.FieldBtnClick(Sender: TObject);
begin
 QueryForm.Show;
 ArrowBtn.Click;
 ArrowBtn.Down := true;
end;

procedure TMainForm.NextReportPageClick(Sender: TObject);
begin
if Assigned(ActiveMDIChild) then
 TMdiChild(ActiveMDIChild).NextPage1click(Sender);
end;

procedure TMainForm.PriviousReportPageClick(Sender: TObject);
begin
if Assigned(ActiveMDIChild) then
 TMdiChild(ActiveMDIChild).PriviousPage1click(Sender);
end;

procedure TMainForm.ZoomClick(Sender: TObject);
begin
ZoomForm.Show;
end;

procedure TMainForm.DatasetsClick(Sender: TObject);
begin
TMDIChild(ActiveMDIChild).EditDatasets;
end;

procedure TMainForm.TextBtnClick(Sender: TObject);
begin
If TSpeedButton(Sender).Down then
   begin
   Screen.Cursor := crDrag;
   if TMdiChild(ActiveMDIChild).Handles.Attached then
      TMdiChild(ActiveMDIChild).Handles.Detach;
   end else
   Screen.Cursor := crDefault;
end;

procedure TMainForm.TextItemClick(Sender: TObject);
begin
 TextBtn.Down := true;
 TextBtnClick(TextBtn);
end;

procedure TMainForm.BandItemClick(Sender: TObject);
begin
 BandBtn.Down := true;
 BandBtnClick(BandBtn);
end;

procedure TMainForm.RefreshBtnClick(Sender: TObject);
begin
 if TMdiChild(ActiveMDIChild).Handles.Attached then
    TMdiChild(ActiveMDIChild).Handles.Detach;
    TMdiChild(ActiveMDIChild).CWReport.ActiveReportPage.Invalidate;
end;

procedure TMainForm.DSBtnClick(Sender: TObject);
begin
 TMDIChild(ActiveMDIChild).EditDatasets;
end;

end.
